"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const language_service_1 = require("@stylable/language-service");
const path_1 = __importDefault(require("path"));
const vscode_uri_1 = require("vscode-uri");
class VscodeStylableLanguageService {
    constructor(connection, docs, fs, stylable) {
        this.languageService = new language_service_1.StylableLanguageService({
            fs,
            stylable
        });
        this.textDocuments = docs;
        this.connection = connection;
    }
    onCompletion(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onCompletion(fsPath, doc.offsetAt(params.position)) : [];
    }
    onDefinition(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onDefinition(fsPath, doc.offsetAt(params.position)) : [];
    }
    onHover(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onHover(fsPath, doc.offsetAt(params.position)) : null;
    }
    onReferences(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onReferences(fsPath, doc.offsetAt(params.position)) : [];
    }
    onRenameRequest(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onRenameRequest(fsPath, doc.offsetAt(params.position), params.newName) : {};
    }
    onSignatureHelp(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc ? this.languageService.onSignatureHelp(fsPath, doc.offsetAt(params.position)) : null;
    }
    onDocumentColor(params) {
        const { fsPath } = vscode_uri_1.URI.parse(params.textDocument.uri);
        return this.languageService.onDocumentColor(fsPath);
    }
    onColorPresentation(params) {
        const { fsPath, doc } = this.getDocAndPath(params.textDocument.uri);
        return doc
            ? this.languageService.onColorPresentation(fsPath, { start: doc.offsetAt(params.range.start), end: doc.offsetAt(params.range.end) }, params.color)
            : [];
    }
    async diagnoseWithVsCodeConfig() {
        let res;
        let ignore = false;
        try {
            res = await this.connection.workspace.getConfiguration({
                section: 'stylable'
            });
            if (!!res && !!res.diagnostics && !!res.diagnostics.ignore && !!res.diagnostics.ignore.length) {
                ignore = true;
            }
        }
        catch (e) {
            /*Client has no workspace/configuration method, ignore silently */
        }
        const result = [];
        this.textDocuments.keys().forEach(key => {
            const doc = this.textDocuments.get(key);
            if (doc) {
                if (doc.languageId === 'stylable') {
                    const uri = vscode_uri_1.URI.parse(doc.uri);
                    // on windows, uri.fsPath replaces separators with '\'
                    // this breaks posix paths in-memory when running on windows
                    // take raw posix path instead
                    const fsPath = uri.scheme === 'file' &&
                        !uri.authority && // not UNC
                        uri.path.charCodeAt(2) !== 58 && // the colon in "c:"
                        path_1.default.isAbsolute(uri.path)
                        ? uri.path
                        : uri.fsPath;
                    let diagnostics;
                    if (ignore &&
                        res.diagnostics.ignore.some(p => {
                            return fsPath.startsWith(p);
                        })) {
                        diagnostics = [];
                    }
                    else {
                        diagnostics = this.languageService.diagnose(fsPath);
                        result.push(...diagnostics);
                    }
                    this.connection.sendDiagnostics({ uri: doc.uri, diagnostics });
                }
            }
        });
        return result;
    }
    onDidClose(event) {
        return this.connection.sendDiagnostics({
            diagnostics: [],
            uri: event.document.uri
        });
    }
    getDocAndPath(uri) {
        const { fsPath } = vscode_uri_1.URI.parse(uri);
        const doc = this.textDocuments.get(uri);
        return { fsPath, doc };
    }
}
exports.VscodeStylableLanguageService = VscodeStylableLanguageService;
//# sourceMappingURL=vscode-service.js.map